# parseHTML.tcl: extract and replace big chunks of HTML files
#
# $Id: parseHTML.tcl,v 1.8 2001/07/22 21:47:10 johnr Exp $
#
# Copyright (c) 1998-2001 The Regents of the University of California.
# All rights reserved. See the file COPYRIGHT for details.
#


### Assumed global variables:
# debug: print debug information if on
# silent: don't print anything if on


#######################################################################
#### quoteAmps
#
# Called an a value passed to regsub as substitution value
#
proc quoteAmps {s} {
    regsub -all {&} $s {\\\&} s
    return $s
}

#######################################################################
#### extractDiv
#
# Get the text between tags like
#
#   <div id="foo"> and
#   </div id="foo">
#
# Note that the id in the end tag is needed for the dumb regexp
# parser to work, although it's not well-formed HTML. Note that the
# text is unmodified.
#
proc extractDiv {textvar id} {
    upvar $textvar string
    if [regexp -nocase "<div id=\"$id\">.*</div id=\"$id\">" \
	    $string result] {
	return $result
    } else {
	return ""
    }
}

#######################################################################
#### substDiv
#
# Substitute the text between tags like
#
#   <div id="foo"> and
#   </div id="foo">
#
# Note that the id in the end tag is needed for the dumb regexp
# parser to work, although it's not well-formed HTML. The text is
# modified with the new value. The return value is 1 if there was
# a substitution, and 0 if the div with the given id was not found.
#
proc substDiv {textvar id value} {
    global debug
    if $debug {
	puts -nonewline "Looking for \"<div id=\"$id\">\""
    }
    upvar $textvar string
    set value [quoteAmps $value]
    if [regsub -nocase "<div id=\"$id\">.*</div id=\"$id\">" \
	    $string \
	    "<div id=\"$id\">\n$value\n</div id=\"$id\">" \
	    string] {
	if $debug {
	    puts " -- found"
	}
	return 1
    } else {
	if $debug {
	    puts " -- not found"
	}
	return 0
    }
}

#######################################################################
#### hasComment
#
# Return 1 if the text has a pair of comments
#
#    <!--foo--> and
#    <!--/foo-->
#
proc hasComment {textvar id} {
    upvar $textvar string
    regexp -nocase "<!--$id-->.*<!--/$id-->" $string
}

#######################################################################
#### extractComment
#
# Get the text between tags like
#
#   <!--foo--> and
#   <!--/foo-->
#
# Note that the text is unmodified.
#
proc extractComment {textvar id} {
    upvar $textvar string
    if [regexp -nocase "<!--$id-->.*<!--/$id-->" \
	    $string result] {
	return $result
    } else {
	return ""
    }
}

#######################################################################
#### substComment
#
# Substitute the text between tags like
#
#   <!--foo--> and
#   <!--/foo-->
#
# The text is
# modified with the new value. The return value is 1 if there was
# a substitution, and 0 if the div with the given id was not found.
#
proc substComment {textvar id value} {
    global debug
    if $debug {
	puts -nonewline "Looking for \"<!--$id-->\""
    }
    upvar $textvar string
    set value [quoteAmps $value]
    if [regsub -nocase "<!--$id-->.*<!--/$id-->" \
	    $string \
	    "<!--$id-->\n$value\n<!--/$id-->" \
	    string] {
	if $debug {
	    puts " -- found"
	}
	return 1
    } else {
	if $debug {
	    puts " -- not found"
	}
	return 0
    }
}

#######################################################################
#### extractTag
#
# Get the text between tags like
#
#   <foo> and
#   </foo>
#
# Note that the text is unmodified.
#
proc extractTag {textvar tag} {
    upvar $textvar string
    if [regexp -nocase "\(<$tag\[^>\]*>\).*</$tag>" \
	    $string result] {
	return $result
    } else {
	return ""
    }
}

#######################################################################
#### substTag
#
# Substitute the text between a pair of open and close tags
#
#   <foo> and
#   </foo>
#
# The text is
# modified with the new value. The return value is 1 if there was
# a substitution, and 0 if the tag pair was not found. NOTE: this
# probably doesn't work if there is more than one occurrence of
# the given tag pair.
#
proc substTag {textvar tag value} {
    global debug
    if $debug {
	puts -nonewline "Looking for \"<$tag>\""
    }
    upvar $textvar string
    set value [quoteAmps $value]
    if [regsub -nocase "\(<$tag\[^>\]*>\).*</$tag>" \
	    $string \
	    "\\1$value</$tag>" \
	    string] {
	if $debug {
	    puts " -- found"
	}
	return 1
    } else {
	if $debug {
	    puts " -- not found"
	}
	return 0
    }
}

#######################################################################
#### substAttribute
#
# Substitute the given attribute of the given tag. Return 1
# if it was substituted, 0 if not.
#
proc substAttribute {textvar tag attr value} {
    global debug
    upvar $textvar string
    set value [quoteAmps $value]
    
    regsub -nocase "<$tag (\[^>\]*)$attr=(\[^ >\]*)(\[^>\]*)>" \
	    $string \
	    "<$tag \\1$attr=\"$value\"\\3>" \
	    string
}

#######################################################################
#### insertAttribute
#
# Insert the given attribute into the given tag. Return 1
# if it was inserted, 0 if not (i.e. the tag was not found).
#
proc insertAttribute {textvar tag attr value} {
    global debug
    upvar $textvar string
    set value [quoteAmps $value]
    
    regsub -nocase "<${tag}(\[^>\]*)>" \
	    $string \
	    "<$tag \\1$attr=\"$value\">" \
	    string
}

#######################################################################
#### hasDiv
#
# Return 1 if the text has a DIV tag pair with the given id, else 0. The text
# is not modified.
#
proc hasDiv {textvar id} {
    upvar $textvar string
    regexp -nocase "<div id=\"$id\">.*</div id=\"$id\">" $string
}

#######################################################################
#### hasTag
#
# Return 1 if the text has the given (open) tag, else 0. The text
# is not modified.
#
proc hasTag {textvar tag} {
    upvar $textvar string
    regexp -nocase "<$tag\[^>\]*>" $string
}

#######################################################################
#### insertBefore
#
# Insert text before the given tag.
#
proc insertBefore {textvar tag insertText} {
    upvar $textvar string
    set insertText [quoteAmps $insertText]
    regsub -nocase "\(<$tag\[^>\]*>\)" $string "$insertText\\1" string
}

#######################################################################
#### insertAfter
#
# Insert text after the given tag.
#
proc insertAfter {textvar tag insertText} {
    upvar $textvar string
    set insertText [quoteAmps $insertText]
    regsub -nocase "\(<$tag\[^>\]*>\)" $string "\\1$insertText" string
}







